import java.io.*;
import java.net.*;
import mpi.*;

/**
 * Captures the clock accuracy, latency and clock skew between two servers.
 * To compile:
 *   mpijavac InfinibandCheckLatency.java  
 * To run:
 *   on server machine enter: mpirun java InfinibandCheckLatency server
 *   on client machine enter: mpirun java InfinibandCheckLatency client <iterations>
 * Always start server machine first
 * When client completes test both client and server will stop.
 *
 */
public class InfinibandCheckLatency {
    private static final String SERVICE_NAME = "InfinibandCheckLatency";
    private static char[] buf = new char[1];

    private static void server() {
        try {
            MPI.Init(new String[0]);
            int rank = MPI.COMM_WORLD.getRank();

            //Server code
            if (rank == 0) {
                String portName = Intracomm.openPort();
                System.out.println("[" + rank + "] portName: " + portName);

                Intracomm.publishName(SERVICE_NAME, portName);
                MPI.COMM_WORLD.barrier();

                Comm client = MPI.COMM_WORLD.accept(portName, 0);
                client.recv(buf, 1, MPI.CHAR, 0, 1);
                client.disconnect();

                Intracomm.closePort(portName);
                Intracomm.unpublishName(SERVICE_NAME, portName);
            }
            
            MPI.Finalize();
        }
        catch (MPIException e) {
            e.printStackTrace();
        }
    }

    private static void client() {
        try {
            MPI.Init(new String[0]);
            int rank = MPI.COMM_WORLD.getRank();

            //Client code
            if (rank == 1) {
                System.out.println("[" + rank + "]");
                //MPI.COMM_WORLD.barrier();
                String portName = Intracomm.lookupName(SERVICE_NAME);
                System.out.println("[" + rank + "] portName: " + portName);

                Comm server = MPI.COMM_SELF.connect(portName, 0);
                server.send(buf, 1, MPI.CHAR, 0, 1);
                server.disconnect();
            }

            MPI.Finalize();
        }
        catch (MPIException e) {
            e.printStackTrace();
        }
    }

    /**
     * Calls clockResolution to determine clock accuracy and then client or server depending on role.
     *
     * @param args - arguments to program
     */
    public static void main(String[] args) {
        String type = args[0];
    
        if (type.equalsIgnoreCase("server")) {
            server();
        }
        else if (type.equalsIgnoreCase("client")) {
            client();
        }
    }
}
